@@ -2,7 +2,6 @@ from base64 import b64encode
 from engineio.json import JSONDecodeError
 import logging
 import queue
-import signal
 import ssl
 import threading
 import time
@@ -16,33 +15,15 @@ try:
     import websocket
 except ImportError:  # pragma: no cover
     websocket = None
+from . import base_client
 from . import exceptions
 from . import packet
 from . import payload
 
 default_logger = logging.getLogger('engineio.client')
-connected_clients = []
 
 
-def signal_handler(sig, frame):
-    """SIGINT handler.
-
-    Disconnect all active clients and then invoke the original signal handler.
-    """
-    for client in connected_clients[:]:
-        if not client.is_asyncio_based():
-            client.disconnect()
-    if callable(original_signal_handler):
-        return original_signal_handler(sig, frame)
-    else:  # pragma: no cover
-        # Handle case where no original SIGINT handler was present.
-        return signal.default_int_handler(sig, frame)
-
-
-original_signal_handler = None
-
-
-class Client(object):
+class Client(base_client.BaseClient):
     """An Engine.IO client.
 
     This class implements a fully compliant Engine.IO web client with support
@@ -75,86 +56,6 @@ class Client(object):
                                     arguments passed to
                                     ``websocket.create_connection()``.
     """
-    event_names = ['connect', 'disconnect', 'message']
-
-    def __init__(self, logger=False, json=None, request_timeout=5,
-                 http_session=None, ssl_verify=True, handle_sigint=True,
-                 websocket_extra_options=None):
-        global original_signal_handler
-        if handle_sigint and original_signal_handler is None and \
-                threading.current_thread() == threading.main_thread():
-            original_signal_handler = signal.signal(signal.SIGINT,
-                                                    signal_handler)
-        self.handlers = {}
-        self.base_url = None
-        self.transports = None
-        self.current_transport = None
-        self.sid = None
-        self.upgrades = None
-        self.ping_interval = None
-        self.ping_timeout = None
-        self.http = http_session
-        self.external_http = http_session is not None
-        self.handle_sigint = handle_sigint
-        self.ws = None
-        self.read_loop_task = None
-        self.write_loop_task = None
-        self.queue = None
-        self.state = 'disconnected'
-        self.ssl_verify = ssl_verify
-        self.websocket_extra_options = websocket_extra_options or {}
-
-        if json is not None:
-            packet.Packet.json = json
-        if not isinstance(logger, bool):
-            self.logger = logger
-        else:
-            self.logger = default_logger
-            if self.logger.level == logging.NOTSET:
-                if logger:
-                    self.logger.setLevel(logging.INFO)
-                else:
-                    self.logger.setLevel(logging.ERROR)
-                self.logger.addHandler(logging.StreamHandler())
-
-        self.request_timeout = request_timeout
-
-    def is_asyncio_based(self):
-        return False
-
-    def on(self, event, handler=None):
-        """Register an event handler.
-
-        :param event: The event name. Can be ``'connect'``, ``'message'`` or
-                      ``'disconnect'``.
-        :param handler: The function that should be invoked to handle the
-                        event. When this parameter is not given, the method
-                        acts as a decorator for the handler function.
-
-        Example usage::
-
-            # as a decorator:
-            @eio.on('connect')
-            def connect_handler():
-                print('Connection request')
-
-            # as a method:
-            def message_handler(msg):
-                print('Received message: ', msg)
-                eio.send('response')
-            eio.on('message', message_handler)
-        """
-        if event not in self.event_names:
-            raise ValueError('Invalid event')
-
-        def set_handler(handler):
-            self.handlers[event] = handler
-            return handler
-
-        if handler is None:
-            return set_handler
-        set_handler(handler)
-
     def connect(self, url, headers=None, transports=None,
                 engineio_path='engine.io'):
         """Connect to an Engine.IO server.
@@ -226,19 +127,11 @@ class Client(object):
                 self.read_loop_task.join()
             self.state = 'disconnected'
             try:
-                connected_clients.remove(self)
+                base_client.connected_clients.remove(self)
             except ValueError:  # pragma: no cover
                 pass
         self._reset()
 
-    def transport(self):
-        """Return the name of the transport currently in use.
-
-        The possible values returned by this function are ``'polling'`` and
-        ``'websocket'``.
-        """
-        return self.current_transport
-
     def start_background_task(self, target, *args, **kwargs):
         """Start a background task.
 
@@ -253,7 +146,8 @@ class Client(object):
         on which the ``join()`` method can be invoked to wait for the task to
         complete.
         """
-        th = threading.Thread(target=target, args=args, kwargs=kwargs)
+        th = threading.Thread(target=target, args=args, kwargs=kwargs,
+                              daemon=True)
         th.start()
         return th
 
@@ -271,10 +165,6 @@ class Client(object):
         """Create an event object."""
         return threading.Event(*args, **kwargs)
 
-    def _reset(self):
-        self.state = 'disconnected'
-        self.sid = None
-
     def _connect_polling(self, url, headers, engineio_path):
         """Establish a long-polling connection to the Engine.IO server."""
         if requests is None:  # pragma: no cover
@@ -319,7 +209,7 @@ class Client(object):
         self.base_url += '&sid=' + self.sid
 
         self.state = 'connected'
-        connected_clients.append(self)
+        base_client.connected_clients.append(self)
         self._trigger_event('connect', run_async=False)
 
         for pkt in p.packets[1:]:
@@ -419,7 +309,10 @@ class Client(object):
                 self.ssl_verify = False
 
         if not self.ssl_verify:
-            extra_options['sslopt'] = {"cert_reqs": ssl.CERT_NONE}
+            if 'sslopt' in extra_options:
+                extra_options['sslopt'].update({"cert_reqs": ssl.CERT_NONE})
+            else:
+                extra_options['sslopt'] = {"cert_reqs": ssl.CERT_NONE}
 
         # combine internally generated options with the ones supplied by the
         # caller. The caller's options take precedence.
@@ -488,7 +381,7 @@ class Client(object):
             self.current_transport = 'websocket'
 
             self.state = 'connected'
-            connected_clients.append(self)
+            base_client.connected_clients.append(self)
             self._trigger_event('connect', run_async=False)
         self.ws = ws
         self.ws.settimeout(self.ping_interval + self.ping_timeout)
@@ -555,31 +448,6 @@ class Client(object):
                 except:
                     self.logger.exception(event + ' handler error')
 
-    def _get_engineio_url(self, url, engineio_path, transport):
-        """Generate the Engine.IO connection URL."""
-        engineio_path = engineio_path.strip('/')
-        parsed_url = urllib.parse.urlparse(url)
-
-        if transport == 'polling':
-            scheme = 'http'
-        elif transport == 'websocket':
-            scheme = 'ws'
-        else:  # pragma: no cover
-            raise ValueError('invalid transport')
-        if parsed_url.scheme in ['https', 'wss']:
-            scheme += 's'
-
-        return ('{scheme}://{netloc}/{path}/?{query}'
-                '{sep}transport={transport}&EIO=4').format(
-                    scheme=scheme, netloc=parsed_url.netloc,
-                    path=engineio_path, query=parsed_url.query,
-                    sep='&' if parsed_url.query else '',
-                    transport=transport)
-
-    def _get_url_timestamp(self):
-        """Generate the Engine.IO query string timestamp."""
-        return '&t=' + str(time.time())
-
     def _read_loop_polling(self):
         """Read packets by polling the Engine.IO server."""
         while self.state == 'connected':
@@ -613,7 +481,7 @@ class Client(object):
         if self.state == 'connected':
             self._trigger_event('disconnect', run_async=False)
             try:
-                connected_clients.remove(self)
+                base_client.connected_clients.remove(self)
             except ValueError:  # pragma: no cover
                 pass
             self._reset()
@@ -655,7 +523,7 @@ class Client(object):
         if self.state == 'connected':
             self._trigger_event('disconnect', run_async=False)
             try:
-                connected_clients.remove(self)
+                base_client.connected_clients.remove(self)
             except ValueError:  # pragma: no cover
                 pass
             self._reset()
